﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using HeuristicLab.Common.Resources;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Persistence.Auxiliary;
namespace HeuristicLab.DebugEngine {

  /// <summary>
  /// Engine espcially for debugging
  /// </summary>
  [View("DebugEngine View")]
  [Content(typeof(DebugEngine), true)]
  public partial class DebugEngineView : ItemView {

    #region Basics

    /// <summary>
    /// Gets or sets the current engine.
    /// </summary>
    /// <remarks>Uses property <see cref="ViewBase.Item"/> of base class <see cref="EditorBase"/>.</remarks>
    public new DebugEngine Content {
      get { return (DebugEngine)base.Content; }
      set { base.Content = value; }
    }

    /// <summary>
    /// Initializes a new instance of <see cref="DebugEngineView"/>.
    /// </summary>
    public DebugEngineView() {
      InitializeComponent();
      updateButton.Image = VS2008ImageLibrary.Refresh;
      stepButton.Image = VS2008ImageLibrary.MoveNext;
      parentButton.Image = VS2008ImageLibrary.ArrowUp;
    }

    /// <summary>
    /// Removes the event handlers from the underlying <see cref="IEngine"/>.
    /// </summary>
    /// <remarks>Calls <see cref="ViewBase.RemoveItemEvents"/> of base class <see cref="ViewBase"/>.</remarks>
    protected override void DeregisterContentEvents() {
      Content.CurrentOperationChanged -= new EventHandler<OperationChangedEventArgs>(Content_CurrentOperationChanged);
      Content.ExecutionTimeChanged -= new EventHandler(Content_ExecutionTimeChanged);
      Content.ExecutionStateChanged -= new EventHandler(Content_ExecutionStateChanged);
      base.DeregisterContentEvents();
    }

    /// <summary>
    /// Adds event handlers to the underlying <see cref="IEngine"/>.
    /// </summary>
    /// <remarks>Calls <see cref="ViewBase.AddItemEvents"/> of base class <see cref="ViewBase"/>.</remarks>
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.ExecutionTimeChanged += new EventHandler(Content_ExecutionTimeChanged);
      Content.ExecutionStateChanged += new EventHandler(Content_ExecutionStateChanged);
      Content.CurrentOperationChanged += new EventHandler<OperationChangedEventArgs>(Content_CurrentOperationChanged);
    }

    /// <summary>
    /// Updates all controls with the latest data of the model.
    /// </summary>
    /// <remarks>Calls <see cref="EditorBase.UpdateControls"/> of base class <see cref="EditorBase"/>.</remarks>
    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        logView.Content = null;
        executionTimeTextBox.Text = "-";
        executionStackView.Content = null;
      } else {
        logView.Content = Content.Log;
        executionTimeTextBox.Text = Content.ExecutionTime.ToString();
        executionStackView.Content = Content.ExecutionStack;
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      if (Content == null) {
        logView.Enabled = false;
        executionTimeTextBox.Enabled = false;
      } else {
        logView.Enabled = true;
        executionTimeTextBox.Enabled = true;
      }
    }

    #endregion

    protected virtual void Content_ExecutionTimeChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_ExecutionTimeChanged), sender, e);
      else
        executionTimeTextBox.Text = Content == null ? "-" : Content.ExecutionTime.ToString();
    }

    void Content_ExecutionStateChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_ExecutionStateChanged), sender, e);
      else {
        switch (Content.ExecutionState) {
          case ExecutionState.Started: executionStackView.SuspendUpdate(); break;
          default: executionStackView.ResumeUpdate(); break;
        }
      }
    }

    #region Current Operation

    void Content_CurrentOperationChanged(object sender, OperationChangedEventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler<OperationChangedEventArgs>(Content_CurrentOperationChanged), sender, e);
      else
        SetOperation(Content.CurrentOperation);
    }

    private void SetOperation(IOperation operation) {
      IAtomicOperation atomicOperation = operation as IAtomicOperation;
      operationTextBox.Text = "<none>";
      parameterCollectionView.Content = null;
      toolTip.SetToolTip(operationTextBox, null);
      if (atomicOperation != null && atomicOperation.Operator != null) {
        operationTextBox.Text = string.Format("Atomic {0}", atomicOperation.Operator.Name);
        toolTip.SetToolTip(operationTextBox, Utils.TypeName(atomicOperation.Operator));
      }
      OperationCollection operationCollection = operation as OperationCollection;
      if (operationCollection != null)
        operationTextBox.Text = string.Format("Collection {0}", operationCollection.Count);
      IExecutionContext context = operation as IExecutionContext;
      IScope scope = null;
      if (context != null) {
        parameterCollectionView.Content = context.Parameters;
        scope = context.Scope;
        while (scope != null && scope.Parent != null)
          scope = scope.Parent;
      }
      UpdateScope(scope);
      if (context != null)
        parentButton.Enabled = context.Parent != null;
      scopeTreeView.Tag = context;
    }

    private void UpdateScope(IScope scope) {
      scopeTreeView.BeginUpdate();
      scopeTreeView.Nodes.Clear();
      if (scope != null) {
        AddScope(scopeTreeView.Nodes, scope);
      }
      scopeTreeView.ExpandAll();
      if (scopeTreeView.Nodes.Count > 0)
        scopeTreeView.TopNode = scopeTreeView.Nodes[0];
      scopeTreeView.EndUpdate();
    }

   

    /*
    private string GetApproximateValue(IParameter param, ref string typeName) {
      string valueString = "<none>";
      IExecutionContext context = Content.CurrentOperation as IExecutionContext;
      IExecutionContext originalContext = param.ExecutionContext;
      object value = null;
      try {
        try {
          param.ExecutionContext = context;
          value = param.ActualValue;
        } finally {
          param.ExecutionContext = originalContext;
        }
      } catch (Exception) { }
      if (value != null) {
        valueString = value.ToString();
        if (context != originalContext)
          valueString = " ~ " + valueString;
        typeName = TypeName(value);
      }
      return valueString;
    }
     */

    private void AddScope(TreeNodeCollection nodes, IScope scope) {
      TreeNode node = nodes.Add(scope.Name);
      if (Content.CurrentAtomicOperation != null && Content.CurrentAtomicOperation.Scope == scope) {
        node.ForeColor = Color.Red;
        node.BackColor = Color.LightGray;
      }
      foreach (var var in scope.Variables) {
        TreeNode varNode = node.Nodes.Add(string.Format("{0}={1}", var.Name, var.Value.ToString()));
        varNode.Tag = var.Value;
        varNode.ToolTipText = Utils.TypeName(var.Value);
      }
      foreach (var subScope in scope.SubScopes) {
        AddScope(node.Nodes, subScope);
      }
    }

    #endregion

    protected virtual void UpdateView() {
      if (InvokeRequired) {
        Invoke(new Action(UpdateView));
      } else {
        SetOperation(Content.CurrentOperation);
      }
    }

    private void stepButton_Click(object sender, EventArgs e) {
      Content.Step();
      UpdateView();
    }

    private void updateButton_Click(object sender, EventArgs e) {
      UpdateView();
    }

    private void scopeTreeView_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e) {
      if (e.Node.Tag != null)
        MainFormManager.MainForm.ShowContent((IItem)e.Node.Tag);
    }

    private void parentButton_Click(object sender, EventArgs e) {
      IExecutionContext context = scopeTreeView.Tag as IExecutionContext;
      if (context != null) {
        SetOperation(context.Parent as IOperation);
      }
    }
  }
}
